@@ -1,10 +1,10 @@
 
-__version__ = "8.20"
+__version__ = "10.00"
 __author__ = "Alex Al-Saffar"
-__date__ = "2022-11-01"
+__date__ = "2023-01-13"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 pypi = r"https://pypi.org/project/crocodile"
 release_notes = """
-Minor fixes
+Added rclone cloud functionality
 """
